@@ -1,5 +1,5 @@
 /*
- * $Id: citadel.c 6281 2008-05-15 17:00:35Z ajc $
+ * $Id: citadel.c 6957 2009-01-16 20:33:00Z ajc $
  *
  * Main source module for the client program.
  */
@@ -1024,7 +1024,7 @@ void system_info(CtdlIPC *ipc)
     scr_printf("Connected users %d / Active users %d / Highest message #%ld\n", mrtg_users, mrtg_active_users, mrtg_himessage);
     scr_printf("Server uptime: %s\n", mrtg_server_uptime);
     scr_printf("Your system administrator is %s.\n", ipc->ServInfo.sysadm);
-    scr_printf("Copyright (C)1987-2008 by the Citadel development team\n");
+    scr_printf("Copyright (C)1987-2009 by the Citadel development team\n");
 }
 
 /*
@@ -1145,6 +1145,14 @@ void get_serv_info(CtdlIPC *ipc, char *supplied_hostname)
 		 /* Look up the , in the bible if you're confused */
 		 (locate_host(ipc, buf), buf), buf);
 
+	/* Indicate to the server that we prefer to decode Base64 and
+	 * quoted-printable on the client side.
+	 */
+	if ((CtdlIPCSpecifyPreferredFormats(ipc, buf, "dont_decode") / 100 ) != 2) {
+		scr_printf("ERROR: Extremely old server; MSG4 framework not supported.\n");
+		logoff(ipc, 0);
+	}
+
 	/*
 	 * Tell the server what our preferred content formats are.
 	 *
@@ -1152,14 +1160,7 @@ void get_serv_info(CtdlIPC *ipc, char *supplied_hostname)
 	 * it to the reader's screen width, but since our HTML-to-text parser
 	 * isn't really all that great, it's probably better to just go with
 	 * the plain text when we have it available.
-	 *
-	 * We also indicate to the server that we prefer to decode Base64 and
-	 * quoted-printable on the client side.
 	 */
-	if ((CtdlIPCSpecifyPreferredFormats(ipc, buf, "dont_decode") / 100 ) != 2) {
-		scr_printf("ERROR: Extremely old server; MSG4 framework not supported.\n");
-		logoff(ipc, 0);
-	}
 	if ((CtdlIPCSpecifyPreferredFormats(ipc, buf, "text/plain|text/html") / 100 ) != 2) {
 		scr_printf("ERROR: Extremely old server; MSG4 framework not supported.\n");
 		logoff(ipc, 0);
@@ -2213,10 +2214,6 @@ NEWUSR:	if (IsEmptyStr(rc_password)) {
 				deletefile(ipc);
 				break;
 
-			case 53:
-				netsendfile(ipc);
-				break;
-
 			case 54:
 				movefile(ipc);
 				break;
