@@ -1,8 +1,8 @@
 /*
- * $Id: citadel.h 6590 2008-09-12 02:39:23Z ajc $
+ * $Id: citadel.h 7525 2009-06-02 19:04:52Z ajc $
  *
  * Main Citadel header file
- * See copyright.txt for copyright information.
+ * See COPYING for copyright information.
  */
 
 /* system customizations are in sysconfig.h */
@@ -38,10 +38,10 @@ extern "C" {
  * usually more strict because you're not really supposed to dump/load and
  * upgrade at the same time.
  */
-#define REV_LEVEL	738		/* This version */
+#define REV_LEVEL	751		/* This version */
 #define REV_MIN		591		/* Oldest compatible database */
 #define EXPORT_REV_MIN	737		/* Oldest compatible export files */
-#define LIBCITADEL_MIN	738		/* Minimum required version of libcitadel */
+#define LIBCITADEL_MIN	744		/* Minimum required version of libcitadel */
 
 #define SERVER_TYPE 0			/* zero for stock Citadel; other developers please
 					   obtain SERVER_TYPE codes for your implementations */
@@ -276,9 +276,9 @@ enum {
 /*
  * Authentication modes
  */
-#define AUTHMODE_NATIVE		0	/* native Citadel authentication */
-#define AUTHMODE_HOST		1	/* host integrated */
-#define AUTHMODE_LDAP		2	/* external LDAP authentication */
+#define AUTHMODE_NATIVE		0	/* Native (self-contained or "black box") */
+#define AUTHMODE_HOST		1	/* Authenticate against the host OS user database */
+#define AUTHMODE_LDAP		2	/* External LDAP authentication (NOT IMPLEMENTED) */
 
 #ifdef __cplusplus
 }
