@@ -1,5 +1,5 @@
 /* 
- * $Id: citserver.c 6452 2008-07-17 19:03:00Z davew $
+ * $Id: citserver.c 7290 2009-04-06 12:14:42Z dothebart $
  *
  * Main source module for the Citadel server
  *
@@ -228,6 +228,8 @@ void master_cleanup(int exitcode) {
 	
 	if (restart_server != 0)
 		exit(1);
+	if ((running_as_daemon != 0) && (exitcode == 0))
+		exitcode = CTDLEXIT_SHUTDOWN;
 	exit(exitcode);
 }
 
@@ -817,16 +819,6 @@ void cmd_ipgm(char *argbuf)
 		CtdlLogPrintf(CTDL_ERR, "Warning: ipgm authentication failed.\n");
 		CC->kill_me = 1;
 	}
-
-	/* Now change the ipgm secret for the next round.
-	 * (Disabled because it breaks concurrent scripts.  The fact that
-	 * we no longer accept IPGM over the network should be sufficient
-	 * to prevent brute-force attacks.  If you don't agree, uncomment
-	 * this block.)
-	get_config();
-	config.c_ipgm_secret = rand();
-	put_config();
-	*/
 }
 
 
@@ -844,18 +836,20 @@ void cmd_down(char *argbuf) {
 		restart_server = extract_int(argbuf, 0);
 		
 		if (restart_server > 0)
-			Reply = "%d Restarting server.  See you soon.\n";
+		{
+			Reply = "%d citserver will now shut down and automatically restart.\n";
+		}
 		if ((restart_server > 0) && !running_as_daemon)
 		{
-			CtdlLogPrintf(CTDL_ERR, "The user requested restart, but not running as deamon! Geronimooooooo!\n");
-			Reply = "%d Warning, not running in deamon mode. maybe we will come up again, but don't lean on it.\n";
+			CtdlLogPrintf(CTDL_ERR, "The user requested restart, but not running as daemon! Geronimooooooo!\n");
+			Reply = "%d Warning: citserver is not running in daemon mode and is therefore unlikely to restart automatically.\n";
 			state = ERROR;
 		}
 		cprintf(Reply, state);
 	}
 	else
 	{
-		cprintf(Reply, CIT_OK + SERVER_SHUTTING_DOWN);
+		cprintf(Reply, CIT_OK + SERVER_SHUTTING_DOWN); 
 	}
 	CtdlThreadStopAll();
 }
@@ -1292,10 +1286,6 @@ void do_command_loop(void) {
 		cmd_movf(&cmdbuf[5]);
 	}
 
-	else if (!strncasecmp(cmdbuf,"NETF",4)) {
-		cmd_netf(&cmdbuf[5]);
-	}
-
 	else if (!strncasecmp(cmdbuf,"OPEN",4)) {
 		cmd_open(&cmdbuf[5]);
 	}
