@@ -1,5 +1,5 @@
 /*
- * $Id: commands.c 6016 2008-02-05 21:13:44Z ajc $
+ * $Id: commands.c 6693 2008-10-16 16:26:33Z ajc $
  *
  * This file contains functions which implement parts of the
  * text-mode user interface.
@@ -1645,7 +1645,7 @@ char keymenu(char *menuprompt, char *menustring) {
 	choices = num_tokens(menustring, '|');
 
 	if (menuprompt != NULL) do_prompt = 1;
-	if (menuprompt != NULL) if (IsEmptyStr(menuprompt)) do_prompt = 0;
+	if ((menuprompt != NULL) && (IsEmptyStr(menuprompt))) do_prompt = 0;
 
 	while (1) {
 		if (display_prompt) {
