@@ -1,5 +1,5 @@
 /*
- * $Id: control.c 6388 2008-06-16 13:06:14Z davew $
+ * $Id: control.c 7380 2009-04-19 18:24:56Z dothebart $
  *
  * This module handles states which are global to the entire server.
  *
@@ -29,6 +29,7 @@
 #include <limits.h>
 #include <sys/types.h>
 #include <sys/file.h>
+#include <sys/stat.h>
 #include <libcitadel.h>
 #include "citadel.h"
 #include "server.h"
@@ -164,6 +165,8 @@ void get_control(void)
 		if (control_fp != NULL) {
 			lock_control();
 			fchown(fileno(control_fp), config.c_ctdluid, -1);
+			fchmod(fileno(control_fp), 
+			       S_IRUSR|S_IWUSR);
 		}
 	}
 	if (control_fp == NULL) {
@@ -171,6 +174,8 @@ void get_control(void)
 		if (control_fp != NULL) {
 			lock_control();
 			fchown(fileno(control_fp), config.c_ctdluid, -1);
+			fchmod(fileno(control_fp), 
+			       S_IRUSR|S_IWUSR);
 			memset(&CitControl, 0, sizeof(struct CitControl));
 			fwrite(&CitControl, sizeof(struct CitControl),
 			       1, control_fp);
@@ -188,6 +193,7 @@ void get_control(void)
 	fread(&CitControl, sizeof(struct CitControl), 1, control_fp);
 	already_have_control = 1;
 	chown(file_citadel_control, config.c_ctdluid, (-1));
+	
 }
 
 /*
@@ -383,6 +389,7 @@ void cmd_conf(char *argbuf)
 		cprintf("%d\n", config.c_xmpp_s2s_port);
 		cprintf("%ld\n", config.c_pop3_fetch);
 		cprintf("%ld\n", config.c_pop3_fastest);
+		cprintf("%d\n", config.c_spam_flag_only);
 		cprintf("000\n");
 	}
 
@@ -636,6 +643,9 @@ void cmd_conf(char *argbuf)
 			case 65:
 				config.c_pop3_fastest = atol(buf);
 				break;
+			case 66:
+				config.c_spam_flag_only = atoi(buf);
+				break;
 			}
 			++a;
 		}
