@@ -396,3 +396,12 @@ int main(int argc, char **argv)
 	}
 	return 0;
 }
+
+
+/*
+ * Stub function
+ */
+void stty_ctdl(int cmd) {
+}
+
+
