@@ -1,5 +1,5 @@
 /* 
- * $Id: ical_dezonify.c 6597 2008-09-13 04:02:20Z ajc $ 
+ * $Id: ical_dezonify.c 6761 2008-11-25 17:37:10Z ajc $ 
  *
  * Function to go through an ical component set and convert all non-UTC
  * date/time properties to UTC.  It also strips out any VTIMEZONE
@@ -16,7 +16,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
-#include <ical.h>
+#include <libical/ical.h>
 #include <libcitadel.h>
 #include "citadel.h"
 #include "server.h"
@@ -88,10 +88,22 @@ void ical_dezonify_backend(icalcomponent *cal,
 				/* CtdlLogPrintf(9, "                * ...and we handle that internally.\n"); */
 			}
 			else {
+				/* try attached first */
 				t = icalcomponent_get_timezone(cal, tzid);
-				/* CtdlLogPrintf(9, "                * ...and I %s have tzdata for that zone.\n",
+/*
+				lprintf(9, "                * ...and I %s have tzdata for that zone.\n",
 					(t ? "DO" : "DO NOT")
-				); */
+				);
+*/
+				/* then try built-in timezones */
+				if (!t) {
+					t = icaltimezone_get_builtin_timezone(tzid);
+/*
+					if (t) {
+						lprintf(9, "                * Using system tzdata!\n");
+					}
+*/
+				}
 			}
 		}
 
