@@ -1,4 +1,4 @@
-/* $Id: ctdl_module.h 6334 2008-05-28 00:40:47Z davew $ */
+/* $Id: ctdl_module.h 6452 2008-07-17 19:03:00Z davew $ */
 
 #ifndef CTDL_MODULE_H
 #define CTDL_MODULE_H
@@ -147,8 +147,12 @@ void CtdlThreadAllocTSD(void);
  * You must free the returned pointer when done.
  */
 struct CitContext *CtdlGetContextArray (int *count);
-void CtdlFillPrivateContext(struct CitContext *context, char *name);
+void CtdlFillSystemContext(struct CitContext *context, char *name);
 
+int CtdlTrySingleUser(void);
+void CtdlEndSingleUser(void);
+int CtdlWantSingleUser(void);
+int CtdlIsSingleUser(void);
 
 
 /*
