@@ -1,8 +1,8 @@
 /*
- * $Id: messages.c 6202 2008-04-18 03:02:12Z ajc $
+ * $Id: messages.c 7199 2009-03-08 14:40:47Z dothebart $
  *
  * Citadel message support routines
- * see copyright.txt for copyright information
+ * see COPYING for copyright information
  */
 
 #include "sysdep.h"
@@ -571,7 +571,7 @@ int read_message(CtdlIPC *ipc,
 
 	/* Set the reply-to address to an Internet e-mail address if possible
 	 */
-	if (message->email != NULL) if (!IsEmptyStr(message->email)) {
+	if ((message->email != NULL) && (!IsEmptyStr(message->email))) {
 		if (!IsEmptyStr(message->author)) {
 			snprintf(reply_to, sizeof reply_to, "%s <%s>", message->author, message->email);
 		}
