@@ -1,5 +1,5 @@
 /*
- * $Id: serv_expire.c 6571 2008-09-07 14:49:53Z davew $
+ * $Id: serv_expire.c 7024 2009-01-30 18:02:22Z ajc $
  *
  * This module handles the expiry of old messages and the purging of old users.
  *
@@ -530,7 +530,8 @@ int PurgeUsers(void) {
 			ForEachUser(do_uid_user_purge, NULL);
 			break;
 		default:
-			CtdlLogPrintf(CTDL_DEBUG, "Unknown authentication mode!\n");
+			CtdlLogPrintf(CTDL_DEBUG, "User purge for auth mode %d is not implemented.\n",
+				config.c_auth_mode);
 			break;
 	}
 
@@ -802,7 +803,7 @@ int PurgeStaleOpenIDassociations(void) {
 	char *deleteme = NULL;
 	long len;
 	void *Value;
-	char *Key;
+	const char *Key;
 	int num_deleted = 0;
 
 	keys = NewHash(1, NULL);
@@ -824,7 +825,7 @@ int PurgeStaleOpenIDassociations(void) {
 
 	/* Go through the hash list, deleting keys we stored in it */
 
-	HashPos = GetNewHashPos();
+	HashPos = GetNewHashPos(keys, 0);
 	while (GetNextHashPos(keys, HashPos, &len, &Key, &Value)!=0)
 	{
 		CtdlLogPrintf(CTDL_DEBUG, "Deleting associated OpenID <%s>\n", Value);
@@ -935,6 +936,10 @@ void *purge_databases(void *args)
 /*****************************************************************************/
 
 
+/* The FSCK command has been removed because people were misusing it */
+
+#if 0
+
 void do_fsck_msg(long msgnum, void *userdata) {
 	struct ctdlroomref *ptr;
 
@@ -1013,6 +1018,7 @@ void cmd_fsck(char *argbuf) {
 
 }
 
+#endif	/* end of commented-out fsck cmd */
 
 /*
  * Manually initiate a run of The Dreaded Auto-Purger (tm)
@@ -1030,11 +1036,11 @@ CTDL_MODULE_INIT(expire)
 {
 	if (!threading)
 	{
-		CtdlRegisterProtoHook(cmd_fsck, "FSCK", "Check message ref counts");
+		/* CtdlRegisterProtoHook(cmd_fsck, "FSCK", "Check message ref counts"); */
 		CtdlRegisterProtoHook(cmd_tdap, "TDAP", "Manually initiate auto-purger");
 	}
 	else
 		CtdlThreadCreate("Auto Purger", CTDLTHREAD_BIGSTACK, purge_databases, NULL);
 	/* return our Subversion id for the Log */
-	return "$Id: serv_expire.c 6571 2008-09-07 14:49:53Z davew $";
+	return "$Id: serv_expire.c 7024 2009-01-30 18:02:22Z ajc $";
 }
