@@ -1,5 +1,5 @@
 /*
- * $Id: imap_fetch.c 6324 2008-05-24 14:37:02Z dothebart $
+ * $Id: imap_fetch.c 7522 2009-06-01 16:44:38Z ajc $
  *
  * Implements the FETCH command in IMAP.
  * This is a good example of the protocol's gratuitous complexity.
@@ -174,7 +174,7 @@ void imap_fetch_rfc822(long msgnum, char *whichfmt) {
 		CC->redirect_len = 0;
 		CC->redirect_alloc = SIZ;
 		CtdlOutputMsg(msgnum, MT_RFC822,
-			(need_body ? HEADERS_ALL : HEADERS_ONLY),
+			(need_body ? HEADERS_ALL : HEADERS_FAST),
 			      0, 1, NULL, 0);
 		if (!need_body) cprintf("\r\n");	/* extra trailing newline */
 		IMAP->cached_rfc822_data = CC->redirect_buffer;
@@ -261,7 +261,7 @@ void imap_fetch_rfc822(long msgnum, char *whichfmt) {
  */
 void imap_load_part(char *name, char *filename, char *partnum, char *disp,
 		    void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
-		    void *cbuserdata)
+		    char *cbid, void *cbuserdata)
 {
 	char mbuf2[SIZ];
 	char *desired_section;
@@ -348,7 +348,7 @@ void imap_output_envelope_from(struct CtdlMessage *msg) {
 
 /*
  * Output an envelope address (or set of addresses) in the official,
- * convuluted, braindead format.  (Note that we can't use this for
+ * convoluted, braindead format.  (Note that we can't use this for
  * the "From" address because its data may come from a number of different
  * fields.  But we can use it for "To" and possibly others.
  */
@@ -530,7 +530,8 @@ void imap_strip_headers(char *section) {
 
 	ptr = CC->redirect_buffer;
 	ok = 0;
-	while ( (done_headers == 0) && (ptr = memreadline(ptr, buf, sizeof buf), *ptr != 0) ) {
+	do {
+		ptr = memreadline(ptr, buf, sizeof buf);
 		if (!isspace(buf[0])) {
 			ok = 0;
 			if (doing_headers == 0) ok = 1;
@@ -556,7 +557,8 @@ void imap_strip_headers(char *section) {
 		if (IsEmptyStr(buf)) done_headers = 1;
 		if (buf[0]=='\r') done_headers = 1;
 		if (buf[0]=='\n') done_headers = 1;
-	}
+		if (*ptr == 0) done_headers = 1;
+	} while (!done_headers);
 
 	strcat(boiled_headers, "\r\n");
 
@@ -591,19 +593,6 @@ void imap_fetch_body(long msgnum, char *item, int is_peek) {
 		section, 
 		IsEmptyStr(section) ? "(empty)" : "");
 
-	/*
-	 * We used to have this great optimization in place that would avoid
-	 * fetching the entire RFC822 message from disk if the client was only
-	 * asking for the headers.  Unfortunately, fetching only the Citadel
-	 * headers omits "Content-type:" and this behavior breaks the iPhone
-	 * email client.  So we have to fetch the whole message from disk.  The
-	 *
-	 *	if (!strncasecmp(section, "HEADER", 6)) {
-	 *		need_body = 0;
-	 *	}
-	 *
-	 */
-
 	/* Burn the cache if we don't have the same section of the 
 	 * same message again.
 	 */
@@ -663,6 +652,10 @@ void imap_fetch_body(long msgnum, char *item, int is_peek) {
 	 * fields, strip it down.
 	 */
 	else if (!strncasecmp(section, "HEADER", 6)) {
+		/* This used to work with HEADERS_FAST, but then Apple got stupid with their
+		 * IMAP library and this broke Mail.App and iPhone Mail, so we had to change it
+		 * to HEADERS_ONLY so the trendy hipsters with their iPhones can read mail.
+		 */
 		CtdlOutputPreLoadedMsg(msg, MT_RFC822, HEADERS_ONLY, 0, 1, 0);
 		imap_strip_headers(section);
 	}
@@ -728,7 +721,7 @@ void imap_fetch_body(long msgnum, char *item, int is_peek) {
 void imap_fetch_bodystructure_pre(
 		char *name, char *filename, char *partnum, char *disp,
 		void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
-		void *cbuserdata
+		char *cbid, void *cbuserdata
 		) {
 
 	cprintf("(");
@@ -742,7 +735,7 @@ void imap_fetch_bodystructure_pre(
 void imap_fetch_bodystructure_post(
 		char *name, char *filename, char *partnum, char *disp,
 		void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
-		void *cbuserdata
+		char *cbid, void *cbuserdata
 		) {
 
 	char subtype[128];
@@ -768,7 +761,7 @@ void imap_fetch_bodystructure_post(
 void imap_fetch_bodystructure_part(
 		char *name, char *filename, char *partnum, char *disp,
 		void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
-		void *cbuserdata
+		char *cbid, void *cbuserdata
 		) {
 
 	int have_cbtype = 0;
@@ -846,12 +839,11 @@ void imap_fetch_bodystructure_part(
 	/* More gratuitous complexity */
 	if ((!strcasecmp(cbmaintype, "MESSAGE"))
 	   && (!strcasecmp(cbsubtype, "RFC822"))) {
-		/* FIXME
-                     A body type of type MESSAGE and subtype RFC822
-                     contains, immediately after the basic fields, the
-                     envelope structure, body structure, and size in
-                     text lines of the encapsulated message.
-		*/
+		/* FIXME: message/rfc822 also needs to output the envelope structure,
+		 * body structure, and line count of the encapsulated message.  Fortunately
+		 * there are not yet any clients depending on this, so we can get away
+		 * with not implementing it for now.
+		 */
 	}
 
 	/* MD5 value of body part; we can get away with NIL'ing this */
@@ -924,12 +916,13 @@ void imap_fetch_bodystructure (long msgnum, char *item,
 		CC->redirect_alloc = 0;
 
 		ptr = rfc822;
-		while (ptr = memreadline(ptr, buf, sizeof buf), *ptr != 0) {
+		do {
+			ptr = memreadline(ptr, buf, sizeof buf);
 			++lines;
 			if ((IsEmptyStr(buf)) && (rfc822_body == NULL)) {
 				rfc822_body = ptr;
 			}
-		}
+		} while (*ptr != 0);
 
 		rfc822_headers_len = rfc822_body - rfc822;
 		rfc822_body_len = rfc822_len - rfc822_headers_len;
