@@ -1,5 +1,5 @@
 /*
- * $Id: serv_imap.c 6453 2008-07-17 19:50:29Z ajc $ 
+ * $Id: serv_imap.c 7291 2009-04-06 14:54:13Z ajc $ 
  *
  * IMAP server for the Citadel system
  * Copyright (C) 2000-2007 by Art Cancro and others.
@@ -447,11 +447,20 @@ void imap_output_capability_string(void) {
 
 	/* We are building a partial implementation of METADATA for the sole purpose
 	 * of interoperating with the ical/vcard version of the Bynari Insight Connector.
-	 * If you were expecting something else, comment out one or both of these
-	 * extension advertisements.
+	 * It is not a full RFC5464 implementation, but it should refuse non-Bynari
+	 * metadata in a compatible and graceful way.
 	 */
 	cprintf(" METADATA");
-	/* cprintf(" LIST-EXTENDED"); */
+
+	/*
+	 * LIST-EXTENDED was originally going to be required by the METADATA extension.
+	 * It was mercifully removed prior to the finalization of RFC5464.  We started
+	 * implementing this but stopped when we learned that it would not be needed.
+	 * If you uncomment this declaration you are responsible for writing a lot of new
+	 * code.
+	 *
+	 * cprintf(" LIST-EXTENDED")
+	 */
 }
 
 /*
@@ -753,7 +762,7 @@ void imap_select(int num_parms, char *parms[])
 	 * 
 	 * Unfortunately, omitting \Deleted as a PERMANENTFLAGS flag causes
 	 * some clients (particularly Thunderbird) to misbehave -- they simply
-	 * electing not to transmit the flag at all.  So we have to advertise
+	 * elect not to transmit the flag at all.  So we have to advertise
 	 * \Deleted as a PERMANENTFLAGS flag, even though it technically isn't.
 	 */
 	cprintf("* FLAGS (\\Deleted \\Seen \\Answered)\r\n");
@@ -1016,7 +1025,7 @@ void imap_status(int num_parms, char *parms[])
 	char savedroom[ROOMNAMELEN];
 	int msgs, new;
 
-	ret = imap_grabroom(roomname, parms[2], 0);
+	ret = imap_grabroom(roomname, parms[2], 1);
 	if (ret != 0) {
 		cprintf
 		    ("%s NO Invalid mailbox name or location, or access denied\r\n",
@@ -1118,7 +1127,7 @@ void imap_unsubscribe(int num_parms, char *parms[])
 	char savedroom[ROOMNAMELEN];
 	int msgs, new;
 
-	ret = imap_grabroom(roomname, parms[2], 0);
+	ret = imap_grabroom(roomname, parms[2], 1);
 	if (ret != 0) {
 		cprintf
 		    ("%s NO Invalid mailbox name or location, or access denied\r\n",
@@ -1632,6 +1641,7 @@ void imap_command_loop(void)
 
 	gettimeofday(&tv2, NULL);
 	total_time = (tv2.tv_usec + (tv2.tv_sec * 1000000)) - (tv1.tv_usec + (tv1.tv_sec * 1000000));
+	CtdlLogPrintf(CTDL_INFO, "IMAP: %s\n", cmdbuf);	// FIXME FIXME FIXME REMOVE THIS NOW
 	CtdlLogPrintf(CTDL_DEBUG, "IMAP command completed in %ld.%ld seconds\n",
 		(total_time / 1000000),
 		(total_time % 1000000)
@@ -1659,5 +1669,5 @@ CTDL_MODULE_INIT(imap)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_imap.c 6453 2008-07-17 19:50:29Z ajc $";
+	return "$Id: serv_imap.c 7291 2009-04-06 14:54:13Z ajc $";
 }
