@@ -1,5 +1,5 @@
 /*
- * $Id: serv_network.c 6381 2008-06-14 14:47:32Z ajc $ 
+ * $Id: serv_network.c 6564 2008-09-02 17:41:36Z ajc $ 
  *
  * This module handles shared rooms, inter-Citadel mail, and outbound
  * mailing list processing.
@@ -478,6 +478,15 @@ void network_deliver_digest(SpoolControl *sc) {
 	msg->cm_fields['F'] = strdup(buf);
 	msg->cm_fields['R'] = strdup(buf);
 
+	/* Set the 'List-ID' header */
+	msg->cm_fields['L'] = malloc(1024);
+	snprintf(msg->cm_fields['L'], 1024,
+		"%s <%ld.list-id.%s>",
+		CC->room.QRname,
+		CC->room.QRnumber,
+		config.c_fqdn
+	);
+
 	/*
 	 * Go fetch the contents of the digest
 	 */
@@ -610,6 +619,18 @@ void network_spool_msg(long msgnum, void *userdata) {
 		msg = CtdlFetchMessage(msgnum, 1);
 		if (msg != NULL) {
 
+			/* Set the 'List-ID' header */
+			if (msg->cm_fields['L'] != NULL) {
+				free(msg->cm_fields['L']);
+			}
+			msg->cm_fields['L'] = malloc(1024);
+			snprintf(msg->cm_fields['L'], 1024,
+				"%s <%ld.list-id.%s>",
+				CC->room.QRname,
+				CC->room.QRnumber,
+				config.c_fqdn
+			);
+
 			/* Prepend "[List name]" to the subject */
 			if (msg->cm_fields['U'] == NULL) {
 				msg->cm_fields['U'] = strdup("(no subject)");
@@ -2083,7 +2104,7 @@ void *network_do_queue(void *args) {
 	struct CitContext networkerCC;
 
 	/* Give the networker its own private CitContext */
-	CtdlFillPrivateContext(&networkerCC, "network");
+	CtdlFillSystemContext(&networkerCC, "network");
 	citthread_setspecific(MyConKey, (void *)&networkerCC );
 
 	/*
@@ -2265,5 +2286,5 @@ CTDL_MODULE_INIT(network)
 	else
 		CtdlThreadSchedule("IGnet Network", CTDLTHREAD_BIGSTACK, network_do_queue, NULL, 0);
 	/* return our Subversion id for the Log */
-	return "$Id: serv_network.c 6381 2008-06-14 14:47:32Z ajc $";
+	return "$Id: serv_network.c 6564 2008-09-02 17:41:36Z ajc $";
 }
