@@ -1,5 +1,5 @@
 /*
- * $Id: serv_openid_rp.c 6495 2008-08-11 19:57:40Z ajc $
+ * $Id: serv_openid_rp.c 6979 2009-01-22 09:48:50Z dothebart $
  *
  * This is an implementation of OpenID 1.1 Relying Party support, in stateless mode.
  *
@@ -131,7 +131,7 @@ void openid_purge(struct ctdluser *usbuf) {
 	char *deleteme = NULL;
 	long len;
 	void *Value;
-	char *Key;
+	const char *Key;
 
 	keys = NewHash(1, NULL);
 	if (!keys) return;
@@ -150,7 +150,7 @@ void openid_purge(struct ctdluser *usbuf) {
 
 	/* Go through the hash list, deleting keys we stored in it */
 
-	HashPos = GetNewHashPos();
+	HashPos = GetNewHashPos(keys, 0);
 	while (GetNextHashPos(keys, HashPos, &len, &Key, &Value)!=0)
 	{
 		CtdlLogPrintf(CTDL_DEBUG, "Deleting associated OpenID <%s>\n", Value);
@@ -541,6 +541,7 @@ int fetch_http(char *url, char *target_buf, int maxbytes, int normalize_len)
 	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, CITADEL);
+	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);		/* die after 180 seconds */
 	if (!IsEmptyStr(config.c_ip_addr)) {
 		curl_easy_setopt(curl, CURLOPT_INTERFACE, config.c_ip_addr);
 	}
@@ -756,6 +757,7 @@ void cmd_oidf(char *argbuf) {
 	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, CITADEL);
+	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);		/* die after 180 seconds */
 	if (!IsEmptyStr(config.c_ip_addr)) {
 		curl_easy_setopt(curl, CURLOPT_INTERFACE, config.c_ip_addr);
 	}
@@ -893,5 +895,5 @@ CTDL_MODULE_INIT(openid_rp)
 	}
 
 	/* return our Subversion id for the Log */
-	return "$Id: serv_openid_rp.c 6495 2008-08-11 19:57:40Z ajc $";
+	return "$Id: serv_openid_rp.c 6979 2009-01-22 09:48:50Z dothebart $";
 }
