@@ -62,6 +62,7 @@ struct rss_item {
 	time_t pubdate;
 	char channel_title[256];
 	int item_tag_nesting;
+	char *author_or_creator;
 };
 
 struct rssnetcfg *rnclist = NULL;
@@ -139,10 +140,20 @@ void rss_save_item(struct rss_item *ri) {
 		msg->cm_magic = CTDLMESSAGE_MAGIC;
 		msg->cm_anon_type = MES_NORMAL;
 		msg->cm_format_type = FMT_RFC822;
-		msg->cm_fields['A'] = strdup("rss");
+
+		if (ri->author_or_creator != NULL) {
+			msg->cm_fields['A'] = html_to_ascii(ri->author_or_creator,
+				strlen(ri->author_or_creator), 512, 0);
+			striplt(msg->cm_fields['A']);
+		}
+		else {
+			msg->cm_fields['A'] = strdup("rss");
+		}
+
 		msg->cm_fields['N'] = strdup(NODENAME);
 		if (ri->title != NULL) {
 			msg->cm_fields['U'] = html_to_ascii(ri->title, strlen(ri->title), 512, 0);
+			striplt(msg->cm_fields['U']);
 		}
 		msg->cm_fields['T'] = malloc(64);
 		snprintf(msg->cm_fields['T'], 64, "%ld", ri->pubdate);
@@ -236,6 +247,8 @@ void rss_xml_start(void *data, const char *supplied_el, const char **attr) {
 		ri->title = NULL;
 		if (ri->link != NULL) free(ri->link);
 		ri->link = NULL;
+		if (ri->author_or_creator != NULL) free(ri->author_or_creator);
+		ri->author_or_creator = NULL;
 		if (ri->description != NULL) free(ri->description);
 		ri->description = NULL;
 
@@ -295,6 +308,12 @@ void rss_xml_end(void *data, const char *supplied_el) {
 		ri->pubdate = rdf_parsedate(ri->chardata);
 	}
 
+	if ( ((!strcasecmp(el, "author")) || (!strcasecmp(el, "creator"))) && (ri->chardata != NULL) ) {
+		if (ri->author_or_creator != NULL) free(ri->author_or_creator);
+		striplt(ri->chardata);
+		ri->author_or_creator = strdup(ri->chardata);
+	}
+
 	if (!strcasecmp(el, "item")) {
 		--ri->item_tag_nesting;
 		rss_save_item(ri);
@@ -381,6 +400,7 @@ void rss_do_fetching(char *url, char *rooms) {
 	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, CITADEL);
+	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);		/* die after 180 seconds */
 	if (!IsEmptyStr(config.c_ip_addr)) {
 		curl_easy_setopt(curl, CURLOPT_INTERFACE, config.c_ip_addr);
 	}
@@ -423,6 +443,8 @@ shutdown:
 	ri.title = NULL;
 	if (ri.link != NULL) free(ri.link);
 	ri.link = NULL;
+	if (ri.author_or_creator != NULL) free(ri.author_or_creator);
+	ri.author_or_creator = NULL;
 	if (ri.description != NULL) free(ri.description);
 	ri.description = NULL;
 	if (ri.chardata_len > 0) {
