@@ -1,5 +1,5 @@
 /*
- * $Id: serv_sieve.c 6324 2008-05-24 14:37:02Z dothebart $
+ * $Id: serv_sieve.c 7221 2009-03-17 15:08:56Z ajc $
  *
  * This module glues libSieve to the Citadel server in order to implement
  * the Sieve mailbox filtering language (RFC 3028).
@@ -508,9 +508,9 @@ void sieve_do_msg(long msgnum, void *userdata) {
 	size_t headers_len = 0;
 	int len = 0;
 
-	if (userdata == NULL)
+	if (u == NULL)
 	{
-		CtdlLogPrintf(CTDL_EMERG, "Cant process Message <%ld>without Userdata!\n", msgnum);
+		CtdlLogPrintf(CTDL_EMERG, "Can't process message <%ld> without userdata!\n", msgnum);
 		return;
 	}
 
@@ -518,11 +518,15 @@ void sieve_do_msg(long msgnum, void *userdata) {
 
 	CtdlLogPrintf(CTDL_DEBUG, "Performing sieve processing on msg <%ld>\n", msgnum);
 
-	msg = CtdlFetchMessage(msgnum, 0);
+	/*
+	 * Make sure you include message body so you can get those second-level headers ;)
+	 */
+	msg = CtdlFetchMessage(msgnum, 1);
 	if (msg == NULL) return;
 
 	/*
 	 * Grab the message headers so we can feed them to libSieve.
+	 * Use HEADERS_ONLY rather than HEADERS_FAST in order to include second-level headers.
 	 */
 	CC->redirect_buffer = malloc(SIZ);
 	CC->redirect_len = 0;
@@ -1305,6 +1309,6 @@ CTDL_MODULE_INIT(sieve)
 	}
 	
         /* return our Subversion id for the Log */
-	return "$Id: serv_sieve.c 6324 2008-05-24 14:37:02Z dothebart $";
+	return "$Id: serv_sieve.c 7221 2009-03-17 15:08:56Z ajc $";
 }
 
