@@ -1,5 +1,5 @@
 /*
- * $Id: serv_vcard.c 6495 2008-08-11 19:57:40Z ajc $
+ * $Id: serv_vcard.c 7214 2009-03-13 02:19:05Z ajc $
  * 
  * A server-side module for Citadel which supports address book information
  * using the standard vCard format.
@@ -87,6 +87,7 @@ void vcard_extract_internet_addresses(struct CtdlMessage *msg,
 				void (*callback)(char *, char *) ) {
 	struct vCard *v;
 	char *s;
+	char *k;
 	char *addr;
 	char citadel_address[SIZ];
 	int instance = 0;
@@ -104,8 +105,9 @@ void vcard_extract_internet_addresses(struct CtdlMessage *msg,
 	 * the "email;internet" key
 	 */
 	do {
-		s = vcard_get_prop(v, "email;internet", 0, instance++, 0);
-		if (s != NULL) {
+		s = vcard_get_prop(v, "email", 1, instance, 0);		/* get any 'email' field */
+		k = vcard_get_prop(v, "email", 1, instance++, 1);	/* but also learn it with attrs */
+		if ( (s != NULL) && (k != NULL) && (bmstrcasestr(k, "internet")) ) {
 			addr = strdup(s);
 			striplt(addr);
 			if (!IsEmptyStr(addr)) {
@@ -247,13 +249,16 @@ void ctdl_vcard_to_directory(struct CtdlMessage *msg, int op) {
 			}
 		}
 
-		if (!strcasecmp(v->prop[i].name, "tel;home"))
+		if ( (!strcasecmp(v->prop[i].name, "tel;home"))
+		   || (!strcasecmp(v->prop[i].name, "tel;type=home")) )
 			(void) CtdlDoDirectoryServiceFunc("homePhone", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
 		else
-		if (!strcasecmp(v->prop[i].name, "tel;fax"))
+		if ( (!strcasecmp(v->prop[i].name, "tel;fax"))
+		   || (!strcasecmp(v->prop[i].name, "tel;type=fax")) )
 			(void) CtdlDoDirectoryServiceFunc("facsimileTelephoneNumber", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
 		else
-		if (!strcasecmp(v->prop[i].name, "tel;cell"))
+		if ( (!strcasecmp(v->prop[i].name, "tel;cell"))
+		   || (!strcasecmp(v->prop[i].name, "tel;type=cell")) )
 			(void) CtdlDoDirectoryServiceFunc("mobile", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
 		else
 		if ( (!strcasecmp(v->prop[i].name, "tel"))
@@ -399,36 +404,39 @@ void cmd_igab(char *argbuf) {
 void extract_inet_email_addrs(char *emailaddrbuf, size_t emailaddrbuf_len,
 				char *secemailaddrbuf, size_t secemailaddrbuf_len,
 				struct vCard *v, int local_addrs_only) {
-	char *s, *addr;
+	char *s, *k, *addr;
 	int instance = 0;
 	int saved_instance = 0;
 
-	/* Go through the vCard searching for *all* instances of
-	 * the "email;internet" key
+	/* Go through the vCard searching for *all* Internet email addresses
 	 */
-	while (s = vcard_get_prop(v, "email;internet", 0, instance++, 0),  s != NULL) {
-		addr = strdup(s);
-		striplt(addr);
-		if (!IsEmptyStr(addr)) {
-			if ( (IsDirectory(addr, 1)) || 
-			     (!local_addrs_only) ) {
-				++saved_instance;
-				if ((saved_instance == 1) && (emailaddrbuf != NULL)) {
-					safestrncpy(emailaddrbuf, addr, emailaddrbuf_len);
-				}
-				else if ((saved_instance == 2) && (secemailaddrbuf != NULL)) {
-					safestrncpy(secemailaddrbuf, addr, secemailaddrbuf_len);
-				}
-				else if ((saved_instance > 2) && (secemailaddrbuf != NULL)) {
-					if ( (strlen(addr) + strlen(secemailaddrbuf) + 2) 
-					   < secemailaddrbuf_len ) {
-						strcat(secemailaddrbuf, "|");
-						strcat(secemailaddrbuf, addr);
+	while (s = vcard_get_prop(v, "email", 1, instance, 0),  s != NULL) {
+		k = vcard_get_prop(v, "email", 1, instance, 1);
+		if ( (s != NULL) && (k != NULL) && (bmstrcasestr(k, "internet")) ) {
+			addr = strdup(s);
+			striplt(addr);
+			if (!IsEmptyStr(addr)) {
+				if ( (IsDirectory(addr, 1)) || 
+			     	(!local_addrs_only) ) {
+					++saved_instance;
+					if ((saved_instance == 1) && (emailaddrbuf != NULL)) {
+						safestrncpy(emailaddrbuf, addr, emailaddrbuf_len);
+					}
+					else if ((saved_instance == 2) && (secemailaddrbuf != NULL)) {
+						safestrncpy(secemailaddrbuf, addr, secemailaddrbuf_len);
+					}
+					else if ((saved_instance > 2) && (secemailaddrbuf != NULL)) {
+						if ( (strlen(addr) + strlen(secemailaddrbuf) + 2) 
+					   	< secemailaddrbuf_len ) {
+							strcat(secemailaddrbuf, "|");
+							strcat(secemailaddrbuf, addr);
+						}
 					}
 				}
 			}
+			free(addr);
 		}
-		free(addr);
+		++instance;
 	}
 }
 
@@ -442,9 +450,9 @@ void extract_friendly_name(char *namebuf, size_t namebuf_len, struct vCard *v)
 {
 	char *s;
 
-	s = vcard_get_prop(v, "fn", 0, 0, 0);
+	s = vcard_get_prop(v, "fn", 1, 0, 0);
 	if (s == NULL) {
-		s = vcard_get_prop(v, "n", 0, 0, 0);
+		s = vcard_get_prop(v, "n", 1, 0, 0);
 	}
 
 	if (s != NULL) {
@@ -458,7 +466,7 @@ void extract_friendly_name(char *namebuf, size_t namebuf_len, struct vCard *v)
  */
 void vcard_extract_vcard(char *name, char *filename, char *partnum, char *disp,
 		   void *content, char *cbtype, char *cbcharset, size_t length,
-		   char *encoding, void *cbuserdata)
+		   char *encoding, char *cbid, void *cbuserdata)
 {
 	struct vCard **v = (struct vCard **) cbuserdata;
 
@@ -542,7 +550,7 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 		return(1);
 	}
 
-	s = vcard_get_prop(v, "FN", 0, 0, 0);
+	s = vcard_get_prop(v, "fn", 1, 0, 0);
 	if (s) CtdlLogPrintf(CTDL_DEBUG, "vCard beforesave hook running for <%s>\n", s);
 
 	if (yes_my_citadel_config) {
@@ -595,7 +603,7 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 	}
 
 	/* If the vCard has no UID, then give it one. */
-	s = vcard_get_prop(v, "UID", 0, 0, 0);
+	s = vcard_get_prop(v, "UID", 1, 0, 0);
 	if (s == NULL) {
 		generate_uuid(buf);
 		vcard_set_prop(v, "UID", buf, 0);
@@ -615,7 +623,7 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 		free(msg->cm_fields['E']);
 		msg->cm_fields['E'] = NULL;
 	}
-	s = vcard_get_prop(v, "UID", 0, 0, 0);
+	s = vcard_get_prop(v, "UID", 1, 0, 0);
 	if (s != NULL) {
 		msg->cm_fields['E'] = strdup(s);
 		if (msg->cm_fields['U'] == NULL) {
@@ -626,9 +634,9 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 	/*
 	 * Set the Subject to the name in the vCard.
 	 */
-	s = vcard_get_prop(v, "FN", 0, 0, 0);
+	s = vcard_get_prop(v, "FN", 1, 0, 0);
 	if (s == NULL) {
-		s = vcard_get_prop(v, "N", 0, 0, 0);
+		s = vcard_get_prop(v, "N", 1, 0, 0);
 	}
 	if (s != NULL) {
 		if (msg->cm_fields['U'] != NULL) {
@@ -666,18 +674,28 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 	long I;
 	struct vCard *v;
 	int is_UserConf=0;
+	int is_MY_UserConf=0;
 	int is_GAB=0;
+	char roomname[ROOMNAMELEN];
 
+	if (msg->cm_format_type != 4) return(0);
 	if (!CC->logged_in) return(0);	/* Only do this if logged in. */
 
-	/* If this isn't the configuration room, or if this isn't a MIME
-	 * message, don't bother.
-	 */
-	if (msg->cm_fields['O'] == NULL) return(0);
-	if (!strcasecmp(msg->cm_fields['O'], USERCONFIGROOM)) is_UserConf = 1;
-	if (!strcasecmp(msg->cm_fields['O'], ADDRESS_BOOK_ROOM)) is_GAB = 1;
+	/* We're interested in user config rooms only. */
+
+	if ( (strlen(CC->room.QRname) >= 12) && (!strcasecmp(&CC->room.QRname[11], USERCONFIGROOM)) ) {
+		is_UserConf = 1;	/* It's someone's config room */
+	}
+	MailboxName(roomname, sizeof roomname, &CC->user, USERCONFIGROOM);
+	if (!strcasecmp(CC->room.QRname, roomname)) {
+		is_UserConf = 1;
+		is_MY_UserConf = 1;	/* It's MY config room */
+	}
+	if (!strcasecmp(CC->room.QRname, ADDRESS_BOOK_ROOM)) {
+		is_GAB = 1;		/* It's the Global Address Book */
+	}
+
 	if (!is_UserConf && !is_GAB) return(0);
-	if (msg->cm_format_type != 4) return(0);
 
 	ptr = msg->cm_fields['M'];
 	if (ptr == NULL) return(0);
@@ -697,12 +715,14 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 			if (I < 0L) return(0);
 
 			/* Store our Internet return address in memory */
-			v = vcard_load(msg->cm_fields['M']);
-			extract_inet_email_addrs(CC->cs_inet_email, sizeof CC->cs_inet_email,
+			if (is_MY_UserConf) {
+				v = vcard_load(msg->cm_fields['M']);
+				extract_inet_email_addrs(CC->cs_inet_email, sizeof CC->cs_inet_email,
 						CC->cs_inet_other_emails, sizeof CC->cs_inet_other_emails,
 						v, 1);
-			extract_friendly_name(CC->cs_inet_fn, sizeof CC->cs_inet_fn, v);
-			vcard_free(v);
+				extract_friendly_name(CC->cs_inet_fn, sizeof CC->cs_inet_fn, v);
+				vcard_free(v);
+			}
 
 			if (!is_GAB)
 			{	// This is not the GAB
@@ -718,8 +738,12 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 			 * But if the user was an Aide or was edited by an Aide then we can
 			 * Assume they don't need validating.
 			 */
-			if (CC->user.axlevel >= 6)
+			if (CC->user.axlevel >= 6) {
+				lgetuser(&CC->user, CC->curr_user);
+				CC->user.flags |= US_REGIS;
+				lputuser(&CC->user);
 				return (0);
+			}
 			
 			set_mm_valid();
 
@@ -875,7 +899,7 @@ void cmd_regi(char *argbuf) {
 				}
 			}
 		}
-		if (a==5) vcard_set_prop(my_vcard, "tel;home", buf, 0);
+		if (a==5) vcard_set_prop(my_vcard, "tel", buf, 0);
 		if (a==6) vcard_set_prop(my_vcard, "email;internet", buf, 0);
 		if (a==7) strcpy(tmpcountry, buf);
 		++a;
@@ -926,10 +950,10 @@ void cmd_greg(char *argbuf)
 	cprintf("%d %s\n", LISTING_FOLLOWS, usbuf.fullname);
 	cprintf("%ld\n", usbuf.usernum);
 	cprintf("%s\n", usbuf.password);
-	s = vcard_get_prop(v, "n", 0, 0, 0);
+	s = vcard_get_prop(v, "n", 1, 0, 0);
 	cprintf("%s\n", s ? s : " ");	/* name */
 
-	s = vcard_get_prop(v, "adr", 0, 0, 0);
+	s = vcard_get_prop(v, "adr", 1, 0, 0);
 	snprintf(adr, sizeof adr, "%s", s ? s : " ");/* address... */
 
 	extract_token(buf, adr, 2, ';', sizeof buf);
@@ -941,7 +965,7 @@ void cmd_greg(char *argbuf)
 	extract_token(buf, adr, 5, ';', sizeof buf);
 	cprintf("%s\n", buf);				/* zip */
 
-	s = vcard_get_prop(v, "tel;home", 0, 0, 0);
+	s = vcard_get_prop(v, "tel", 1, 0, 0);
 	if (s == NULL) s = vcard_get_prop(v, "tel", 1, 0, 0);
 	if (s != NULL) {
 		cprintf("%s\n", s);
@@ -993,8 +1017,9 @@ void vcard_newuser(struct ctdluser *usbuf) {
 #ifdef SOLARIS_GETPWUID
 		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer) != NULL) {
 #else // SOLARIS_GETPWUID
-		struct passwd **result = NULL;
-		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer, result) == 0) {
+		struct passwd *result = NULL;
+		CtdlLogPrintf(CTDL_DEBUG, "Searching for uid %d\n", usbuf->uid);
+		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer, &result) == 0) {
 #endif // HAVE_GETPWUID_R
 			snprintf(buf, sizeof buf, "%s@%s", pwd.pw_name, config.c_fqdn);
 			vcard_add_prop(v, "email;internet", buf);
@@ -1176,7 +1201,7 @@ void cmd_gvea(char *argbuf)
  */
 void dvca_mime_callback(char *name, char *filename, char *partnum, char *disp,
 		void *content, char *cbtype, char *cbcharset, size_t length, char *encoding,
-		void *cbuserdata) {
+		char *cbid, void *cbuserdata) {
 
 	struct vCard *v;
 	char displayname[256];
@@ -1471,7 +1496,7 @@ void store_this_ha(struct addresses_to_be_filed *aptr) {
 			vmsg->cm_anon_type = MES_NORMAL;
 			vmsg->cm_format_type = FMT_RFC822;
 			vmsg->cm_fields['A'] = strdup("Citadel");
-			vmsg->cm_fields['E'] =  strdup(vcard_get_prop(v, "UID", 0, 0, 0));
+			vmsg->cm_fields['E'] =  strdup(vcard_get_prop(v, "UID", 1, 0, 0));
 			ser = vcard_serialize(v);
 			if (ser != NULL) {
 				vmsg->cm_fields['M'] = malloc(strlen(ser) + 1024);
@@ -1602,5 +1627,5 @@ CTDL_MODULE_INIT(vcard)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_vcard.c 6495 2008-08-11 19:57:40Z ajc $";
+	return "$Id: serv_vcard.c 7214 2009-03-13 02:19:05Z ajc $";
 }
