@@ -1,5 +1,5 @@
 /* 
- * $Id: room_ops.c 6146 2008-03-21 21:42:54Z davew $
+ * $Id: room_ops.c 7230 2009-03-19 03:41:51Z ajc $
  * 
  * Server functions which perform operations on room objects.
  *
@@ -181,7 +181,7 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 	if ( (userbuf->axlevel >= 6)
 	   || (userbuf->usernum == roombuf->QRroomaide)
 	   ) {
-		retval = retval | UA_ADMINALLOWED | UA_DELETEALLOWED;
+		retval = retval | UA_ADMINALLOWED | UA_DELETEALLOWED | UA_POSTALLOWED;
 	}
 
 NEWMSG:	/* By the way, we also check for the presence of new messages */
