@@ -1,5 +1,5 @@
 /*
- * $Id: rooms.c 6103 2008-03-10 19:34:08Z dothebart $
+ * $Id: rooms.c 6955 2009-01-16 20:18:18Z ajc $
  *
  * 
  * Client-side functions which perform room operations
@@ -625,11 +625,6 @@ void editthisroom(CtdlIPC *ipc)
 		}
 	} while (r / 100 != 2);
 
-	/* FIXME: Duplicate code??? */
-	if (!strcasecmp(raide, "none")) {
-		strcpy(raide, "");
-	}
-
 	/* Angels and demons dancing in my head... */
 	do {
 		snprintf(buf, sizeof buf, "%d", attr->QRep.expire_mode);
