@@ -1,5 +1,5 @@
 /*
- * $Id: sendcommand.c 6410 2008-06-29 03:18:07Z ajc $
+ * $Id: sendcommand.c 6958 2009-01-16 20:49:12Z ajc $
  *
  * Command-line utility to transmit a server command.
  *
@@ -305,3 +305,11 @@ int main(int argc, char **argv)
 	}
 	return 0;
 }
+
+/*
+ * Stub function
+ */
+void stty_ctdl(int cmd) {
+}
+
+
