@@ -1,4 +1,4 @@
-/* $Id: server.h 6451 2008-07-17 18:39:34Z davew $ */
+/* $Id: server.h 7292 2009-04-06 20:52:56Z ajc $ */
 
 
 #ifndef SERVER_H
@@ -32,6 +32,8 @@ struct CtdlMessage {
 
 
 
+#define CTDLEXIT_SHUTDOWN	0	/* Normal shutdown; do NOT auto-restart */
+
 /*
  * Exit codes 101 through 109 are used for conditions in which
  * we deliberately do NOT want the service to automatically
@@ -46,7 +48,6 @@ struct CtdlMessage {
 
 
 
-
 /*
  * Here's the big one... the Citadel context structure.
  *
@@ -123,10 +124,6 @@ struct CitContext {
 	int redirect_ssl;
 #endif
 
-	int buffering;
-	char *output_buffer;	/* hold output for one big dump */
-	int buffer_len;
-
 	/* A linked list of all instant messages sent to us. */
 	struct ExpressMessage *FirstExpressMessage;
 	int disable_exp;	/* Set to 1 to disable incoming pages */
