@@ -1,7 +1,7 @@
 /*
  * citserver's main() function lives here.
  *
- * $Id: server_main.c 6410 2008-06-29 03:18:07Z ajc $
+ * $Id: server_main.c 7265 2009-03-25 23:18:46Z dothebart $
  */
 
 #include "sysdep.h"
@@ -310,6 +310,14 @@ int main(int argc, char **argv)
 		start_chkpwd_daemon();
 	}
 
+
+	/*
+	 * check, whether we're fired up another time after a crash.
+	 * if, post an aide message, so the admin has a chance to react.
+	 */
+	checkcrash ();
+
+
 	/*
 	 * Now that we've bound the sockets, change to the Citadel user id and its
 	 * corresponding group ids
