@@ -1,8 +1,8 @@
 /*
- * $Id: sysdep.c 6452 2008-07-17 19:03:00Z davew $
+ * $Id: sysdep.c 7333 2009-04-11 21:29:21Z ajc $
  *
  * Citadel "system dependent" stuff.
- * See copyright.txt for copyright information.
+ * See COPYING for copyright information.
  *
  * Here's where we (hopefully) have most parts of the Citadel server that
  * would need to be altered to run the server in a non-POSIX environment.
@@ -380,7 +380,7 @@ int ig_uds_server(char *sockpath, int queue_len, char **errormessage)
 	if (actual_queue_len < 5) actual_queue_len = 5;
 
 	i = unlink(sockpath);
-	if (i != 0) if (errno != ENOENT) {
+	if ((i != 0) && (errno != ENOENT)) {
 		*errormessage = (char*) malloc(SIZ + 1);
 		snprintf(*errormessage, SIZ, "citserver: can't unlink %s: %s",
 			sockpath, strerror(errno));
@@ -548,9 +548,8 @@ void CtdlFillSystemContext(struct CitContext *context, char *name)
 }
 
 /*
- * The following functions implement output buffering. If the kernel supplies
- * native TCP buffering (Linux & *BSD), use that; otherwise, emulate it with
- * user-space buffering.
+ * The following functions implement output buffering on operating systems which
+ * support it (such as Linux and various BSD flavors).
  */
 #ifndef HAVE_DARWIN
 #ifdef TCP_CORK
@@ -563,64 +562,33 @@ void CtdlFillSystemContext(struct CitContext *context, char *name)
 #endif /* TCP_CORK */
 #endif /* HAVE_DARWIN */
 
-#ifdef HAVE_TCP_BUFFERING
 static unsigned on = 1, off = 0;
+
 void buffer_output(void) {
-	struct CitContext *ctx = MyContext();
-	setsockopt(ctx->client_socket, IPPROTO_TCP, TCP_CORK, &on, 4);
-	ctx->buffering = 1;
+#ifdef HAVE_TCP_BUFFERING
+#ifdef HAVE_OPENSSL
+	if (!CC->redirect_ssl)
+#endif
+		setsockopt(CC->client_socket, IPPROTO_TCP, TCP_CORK, &on, 4);
+#endif
 }
 
 void unbuffer_output(void) {
-	struct CitContext *ctx = MyContext();
-	setsockopt(ctx->client_socket, IPPROTO_TCP, TCP_CORK, &off, 4);
-	ctx->buffering = 0;
+#ifdef HAVE_TCP_BUFFERING
+#ifdef HAVE_OPENSSL
+	if (!CC->redirect_ssl)
+#endif
+		setsockopt(CC->client_socket, IPPROTO_TCP, TCP_CORK, &off, 4);
+#endif
 }
 
 void flush_output(void) {
-	struct CitContext *ctx = MyContext();
-	setsockopt(ctx->client_socket, IPPROTO_TCP, TCP_CORK, &off, 4);
-	setsockopt(ctx->client_socket, IPPROTO_TCP, TCP_CORK, &on, 4);
+#ifdef HAVE_TCP_BUFFERING
+	struct CitContext *CCC = CC;
+	setsockopt(CCC->client_socket, IPPROTO_TCP, TCP_CORK, &off, 4);
+	setsockopt(CCC->client_socket, IPPROTO_TCP, TCP_CORK, &on, 4);
+#endif
 }
-#else 
-#ifdef HAVE_DARWIN
-/* Stub functions for Darwin/OS X where TCP buffering isn't liked at all */
-void buffer_output(void) {
-	CC->buffering = 0;
-}
-void unbuffer_output(void) {
-	CC->buffering = 0;
-}
-void flush_output(void) {
-}
-#else
-void buffer_output(void) {
-	if (CC->buffering == 0) {
-		CC->buffering = 1;
-		CC->buffer_len = 0;
-		CC->output_buffer = malloc(SIZ);
-	}
-}
-
-void flush_output(void) {
-	if (CC->buffering == 1) {
-		client_write(CC->output_buffer, CC->buffer_len);
-		CC->buffer_len = 0;
-	}
-}
-
-void unbuffer_output(void) {
-	if (CC->buffering == 1) {
-		CC->buffering = 0;
-		/* We don't call flush_output because we can't. */
-		client_write(CC->output_buffer, CC->buffer_len);
-		CC->buffer_len = 0;
-		free(CC->output_buffer);
-		CC->output_buffer = NULL;
-	}
-}
-#endif /* HAVE_DARWIN */
-#endif /* HAVE_TCP_BUFFERING */
 
 
 
@@ -651,19 +619,6 @@ int client_write(char *buf, int nbytes)
 		return 0;
 	}
 
-#ifndef HAVE_TCP_BUFFERING
-	/* If we're buffering for later, do that now. */
-	if (Ctx->buffering) {
-		old_buffer_len = Ctx->buffer_len;
-		Ctx->buffer_len += nbytes;
-		Ctx->output_buffer = realloc(Ctx->output_buffer, Ctx->buffer_len);
-		memcpy(&Ctx->output_buffer[old_buffer_len], buf, nbytes);
-		return 0;
-	}
-#endif
-
-	/* Ok, at this point we're not buffering.  Go ahead and write. */
-
 #ifdef HAVE_OPENSSL
 	if (Ctx->redirect_ssl) {
 		client_write_ssl(buf, nbytes);
@@ -707,13 +662,12 @@ int client_write(char *buf, int nbytes)
 
 
 /*
- * cprintf()  ...   Send formatted printable data to the client.   It is
- *		  implemented in terms of client_write() but remains in
- *		  sysdep.c in case we port to somewhere without va_args...
+ * cprintf()	Send formatted printable data to the client.
+ *		Implemented in terms of client_write() so it's technically not sysdep...
  */
 void cprintf(const char *format, ...) {   
 	va_list arg_ptr;   
-	char buf[1024];   
+	char buf[1024];
    
 	va_start(arg_ptr, format);   
 	if (vsnprintf(buf, sizeof buf, format, arg_ptr) == -1)
@@ -948,6 +902,9 @@ void graceful_shutdown(int signum) {
 	exit(0);
 }
 
+int nFireUps = 0;
+int nFireUpsNonRestart = 0;
+pid_t ForkedPid = 1;
 
 /*
  * Start running as a daemon.
@@ -1003,31 +960,24 @@ void start_daemon(int unused) {
 			}
 			waitpid(current_child, &status, 0);
 		}
-
 		do_restart = 0;
-
-		/* Did the main process exit with an actual exit code? */
-		if (WIFEXITED(status)) {
-
-			/* Exit code 0 means the watcher should exit */
-			if (WEXITSTATUS(status) == 0) {
-				do_restart = 0;
-			}
-
-			/* Exit code 101-109 means the watcher should exit */
-			else if ( (WEXITSTATUS(status) >= 101) && (WEXITSTATUS(status) <= 109) ) {
-				do_restart = 0;
-			}
-
-			/* Any other exit code means we should restart. */
-			else {
-				do_restart = 1;
-			}
+		nFireUpsNonRestart = nFireUps;
+		
+		/* Exit code 0 means the watcher should exit */
+		if (WIFEXITED(status) && (WEXITSTATUS(status) == CTDLEXIT_SHUTDOWN)) {
+			do_restart = 0;
 		}
 
-		/* Any other type of termination (signals, etc.) should also restart. */
+		/* Exit code 101-109 means the watcher should exit */
+		else if (WIFEXITED(status) && (WEXITSTATUS(status) >= 101) && (WEXITSTATUS(status) <= 109)) {
+			do_restart = 0;
+		}
+
+		/* Any other exit code, or no exit code, means we should restart. */
 		else {
 			do_restart = 1;
+			nFireUps++;
+			ForkedPid = current_child;
 		}
 
 	} while (do_restart);
@@ -1038,6 +988,32 @@ void start_daemon(int unused) {
 
 
 
+void checkcrash(void)
+{
+	if (nFireUpsNonRestart != nFireUps)
+	{
+		StrBuf *CrashMail;
+
+		CrashMail = NewStrBuf();
+		CtdlLogPrintf(CTDL_ALERT, "Posting crash message\n");
+		StrBufPrintf(CrashMail, 
+			" \n"
+			" The Citadel server process (citserver) terminated unexpectedly."
+			"\n \n"
+			" This could be the result of a bug in the server program, or some external "
+			"factor.  You can obtain more information about this by enabling core dumps. "
+			"For more information, please see: "
+			"http://citadel.org/doku.php/faq:mastering_your_os:gdb#how.do.i.make.my.system.produce.core-files"
+			"\n \n"
+			" If you have already done this, the core dump is likely to be found at %score.%d\n"
+			,
+			ctdl_run_dir, ForkedPid);
+		aide_message(ChrPtr(CrashMail), "Citadel server process terminated unexpectedly");
+		FreeStrBuf(&CrashMail);
+	}
+}
+
+
 /*
  * Generic routine to convert a login name to a full name (gecos)
  * Returns nonzero if a conversion took place
