@@ -63,7 +63,7 @@ extern int screenwidth;
 void do_system_configuration(CtdlIPC *ipc)
 {
 
-#define NUM_CONFIGS 65
+#define NUM_CONFIGS 67
 
 	char buf[256];
 	char sc[NUM_CONFIGS][256];
@@ -186,6 +186,10 @@ void do_system_configuration(CtdlIPC *ipc)
 	a = (a ? 0 : 1);
 	snprintf(sc[25], sizeof sc[25], "%d", a);
 
+	snprintf(sc[66], sizeof sc[66], "%d", (boolprompt(
+		"Flag messages as spam instead of rejecting",
+		atoi(&sc[66][0]))));
+
 	/* This logic flips the question around, because it's one of those
 	 * situations where 0=yes and 1=no
 	 */
@@ -390,7 +394,8 @@ void get_inet_rec_type(CtdlIPC *ipc, char *buf) {
 	keyopt(" <4> SpamAssassin   (Address of SpamAssassin server)\n");
 	keyopt(" <5> RBL            (domain suffix of spam hunting RBL)\n");
 	keyopt(" <6> masq domains   (Domains as which users are allowed to masquerade)\n");
-	sel = intprompt("Which one", 1, 1, 6);
+	keyopt(" <7> ClamAV         (Address of ClamAV clamd server)\n");
+	sel = intprompt("Which one", 1, 1, 7);
 	switch(sel) {
 		case 1:	strcpy(buf, "localhost");
 			return;
@@ -404,6 +409,8 @@ void get_inet_rec_type(CtdlIPC *ipc, char *buf) {
 			return;
 		case 6:	strcpy(buf, "masqdomain");
 			return;
+		case 7:	strcpy(buf, "clamav");
+			return;
 	}
 }
 
