@@ -1,5 +1,5 @@
 /* 
- * $Id: user_ops.c 6447 2008-07-17 12:57:23Z davew $
+ * $Id: user_ops.c 7211 2009-03-12 15:12:32Z ajc $
  *
  * Server functions which perform operations on user objects.
  *
@@ -728,9 +728,7 @@ void logout(void)
 		purge_user(CCC->user.fullname);
 
 	/* Free any output buffers */
-	if (CCC->output_buffer != NULL) {
-		unbuffer_output();
-	}
+	unbuffer_output();
 }
 
 /*
@@ -1062,14 +1060,7 @@ int create_user(char *newusername, int become_user)
 			uid = pd.pw_uid;
 			if (IsEmptyStr (username))
 			{
-				CtdlLogPrintf (CTDL_EMERG, 
-					 "Can't find Realname for user %s [%d] in the Host Auth Database; giving up.\n", 
-					 newusername, pd.pw_uid);
-				snprintf(buf, SIZ, 
-					 "Can't find Realname for user %s [%d] in the Host Auth Database; giving up.\n",
-					 newusername, pd.pw_uid);
-				aide_message(buf, "User Creation Failure Notice");
-
+				safestrncpy(username, pd.pw_name, sizeof username);
 			}
 		}
 		else {
